<?php
//IMPORT LIBRARIES ------------------------------------------------------
include $_SERVER["DOCUMENT_ROOT"] . "/frifacturation/server/BVR_Reader/vendor/autoload.php";
//-----------------------------------------------------------------------

//PROCESS THE FILE ------------------------------------------------------
function BvrReader($filename)
{
    //GET THE FILE ------------------------------------------------------
    // $filename = $_POST['file'];
    $BvrV11 = new \Ticketpark\v11\v11($_SERVER["DOCUMENT_ROOT"] . '/frifacturation/server/BVR_Reader/files/'.$filename);
    //-------------------------------------------------------------------

    //VALIDATE THE CONTENT OF THE FILE ----------------------------------
    if(!$BvrV11->validate()) {
        $response["errors"] = true;
        $response["type"] = "file";
        $response["file"] = true;
        $response["line"] = "FAT0001";
        $response["message"] = $BvrV11->getError();
        // die(json_encode($response));
        return $response;
    }
    //-------------------------------------------------------------------

    //DEFINE VARS -------------------------------------------------------
    $controlTotal = 0;
    $resumeLineTotal = 0;
    $rowsRecords = array();
    $resumeRecord = array();
    //-------------------------------------------------------------------

    //GET THE LINES RECORDS (ONLY ROWS) (more info/methods: Ticketpark\v11\Record\TransactionRecord)
    foreach($BvrV11->getTransactionRecords() as $record){
        $rowsRecords[] = array(
                //Transactions
                'Transaction_Code' => $record->getTransactionCode()->getTransactionCode(),
                'Transaction_Type' => $record->getTransactionCode()->getTransactionType(),
                //Banking
                'Participant_ID' => $record->getParticipantIdentifier(),
                'Banking_Account' => $record->getBankingAccount(),
                'Internal_Bank_Reference' => $record->getInternalBankReference(),
                'Microfilm_Reference' => $record->getMicrofilmReference(),
                //Payment
                'Payment_type' => $record->getTransactionCode()->getPaymentType(),
                'Signed_Amount' => $record->getSignedAmount(),
                'ABS_Amount' => $record->getAmount(),
                //Dates
                'Date_Paid' => (array) $record->getDatePaid(),
                'Date_Processed' => (array) $record->getDateProcessed(),
                'Date_Credit_Note' => (array) $record->getDateCreditNote(),
                'DateValuta' => (array) $record->getDateValuta(),
                //BVR References
                'Full_BVR' => $record->getReferenceNumber(),
                'Short_BVR' => $record->getCustomReferenceNumber(),
                //Other
                'Reject_Code' => $record->getRejectCode(),
                'Fee' => $record->getFee(),
            );
        //check the total control
        $controlTotal += $record->getSignedAmount();    
    }
    //-------------------------------------------------------------------

    //GET THE RESUME LINE (LAST LINE) (more info/methods: Ticketpark\v11\Record\TransactionRecord)
    $recordLast = $BvrV11->getTotalRecord();
    $resumeLineTotal = $recordLast->getSignedAmount();

    $resumeRecord[] = array(
        //Transactions
        'Transaction_Code' => $recordLast->getTransactionCode()->getTransactionCode(),
        'Transaction_Type' => $recordLast->getTransactionCode()->getTransactionType(),
        'Number_Transactions' => $recordLast->getNumberOfTransactions(),
        //Banking
        'Banking_Account' => $recordLast->getBankingAccount(),
        //Payment
        'Signed_Amount' => $recordLast->getSignedAmount(),
        'ABS_Amount' => $recordLast->getAmount(),
        'Date_File_Creation' => (array) $recordLast->getDateFileCreation(),
        //Other
        'Sorting_Key' => $recordLast->getSortingKey(),
        'Total_Fees' => $recordLast->getTotalFees(),
    );
    //-------------------------------------------------------------------

    //VALIDATE TOTALS ---------------------------------------------------
    $controlTotal = floatval($controlTotal);    
    $resumeLineTotal = floatval($resumeLineTotal);
    
    if(bccomp($controlTotal, $resumeLineTotal) != 0) {
        $response["errors"] = true;
        $response["type"] = "var";
        $response["totals"] = true;
        $response["line"] = "FAT0002";
        $response["message"] = "Valores diferentes: control=".$controlTotal.", resume=".$resumeLineTotal;
        // die(json_encode($response));
        return $response;
    }
    //-------------------------------------------------------------------
    
    //RETURN CONTENT
    $response["errors"] = false;
    $response["lines"] = $rowsRecords;
    $response["resume"] = $resumeRecord;
    return $response;
}
//-----------------------------------------------------------------------

// var_dump(BvrReader('061017-01.v11'));